
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <errno.h>
#include <sys/select.h>
#include <time.h>
#include <signal.h>

#include "ezxml/ezxml.h"

#define XML_BUFFER_SIZE 0x2000
#define XML_READ_TIMEOUT 1000

#if 1
#define DEBUG(x) printf x
#else
#define DEBUG(x)
#endif 

static int intExitLoop = 0;



void exit_program(int sig)
{
	DEBUG(("exiting...\r\n"));
	intExitLoop = 1;
}


static int parseXML(char *pcaData, size_t len, char *pcaLogFile )
{
    int intReturn =0;

    ezxml_t xmlFile = ezxml_parse_str(pcaData, len);

    if (xmlFile==NULL)
    {
        DEBUG(("failed to read XML file\r\n"));
    }
    else
    {
        // get the error code
        const char *pcaError = ezxml_error(xmlFile);

        // the error code is a string, check 
        // that the error string is empty                                                                  
        if (strcmp(pcaError, "") != 0)
        {
            // error
            DEBUG(("xml loading error: %s\n", pcaError));        
        }
        else
        {
			// try to open the log file
			FILE *logFile = fopen(pcaLogFile, "a+");


			if (logFile==NULL)
			{
				DEBUG(("failed to open log file %s: %s\n", pcaLogFile,strerror(errno)));        
			}
			else
			{
				ezxml_t xmlChannel, xmlDate;
				char pcaBuff[200];

				// get the time and date
				// <date><dsb>00004</dsb><hr>13</hr><min>08</min><sec>49</sec></date>
		
				// output as:
				// date +"%m/%d/%y %H:%M:%S"`
		
				time_t tmtTime; 
				// first we need to get the current time
				tmtTime = time(NULL);
		
				strftime(pcaBuff, sizeof(pcaBuff), "%m/%d/%y %H:%M:%S", gmtime(&tmtTime) );
		
				fprintf(logFile,"%s",pcaBuff );
		
				xmlChannel = ezxml_child(xmlFile, "ch1");
				fprintf(logFile, ", %s", ezxml_child(xmlChannel, "watts")->txt);
				
				xmlChannel = ezxml_child(xmlFile, "ch2");
				fprintf(logFile, ", %s", ezxml_child(xmlChannel, "watts")->txt);
					
				xmlChannel = ezxml_child(xmlFile, "ch3");
				fprintf(logFile, ", %s", ezxml_child(xmlChannel, "watts")->txt);            
		
				fprintf(logFile, ", %s\r\n", ezxml_child(xmlFile, "tmpr")->txt);
		
				fclose(logFile);

				intReturn = 1;
			}
    
            ezxml_free(xmlFile);
        }
    }

    return intReturn;
}


static int readXML(int intSerialFD, char *pcaBuffer, int intMaxSize)
{
    int intReturn =0;

    int intSelectResponce;
    fd_set fbsFileSelect;
    struct timeval timeout;    

    tcflush(intSerialFD, TCIFLUSH);

    // wait for up to 6 seconds to start

    intReturn = read( intSerialFD, pcaBuffer, intMaxSize );    


    return intReturn;    
}


static int openSerial(char *pcaSerialPort)
{
	int intSerialFD;
	int intReturn =-1; 

	// try to open the serial port
	intSerialFD = open(pcaSerialPort,  O_RDONLY );
	
	// ok, now we need to check that the device was opened
	if (intSerialFD < 0)
	{
		DEBUG(("failed to open serial port\n"));
	}
	else
	{
		struct termios comm_state;
		
		memset(&comm_state, 0x0, sizeof(comm_state));
	
		comm_state.c_cflag = CS8 | CLOCAL | CREAD; // 8 bit mode
	
		comm_state.c_lflag = ICANON;
	
		comm_state.c_cc[VMIN] = 1;
		comm_state.c_cc[VTIME] = 1;
		
		cfsetospeed(&comm_state, B9600);
		cfsetispeed(&comm_state, B9600);
		tcflush(intSerialFD, TCIFLUSH);
		tcsetattr(intSerialFD, TCSANOW, &comm_state);

		intReturn=intSerialFD;
	}

	return(intReturn);
}

int main(int argc, char **argv )
{
    int intReturn = 1;


    // start by reading the serial port
    // fist open it
	DEBUG(("starting %s\r\n", argv[0]));

    if (argc != 3)
    {
        printf("usage: %s <SerialPort> <OutputLog>\n", argv[0]);
    }
    else
    {
        int intSerialFD;    
		intSerialFD = openSerial(argv[1]);
		if (intSerialFD<0)
		{
			DEBUG(("failed to open serial port %s\r\n",argv[1] ));
		}
		else
		{

            // allocate memory for XML data
            char *pcaBuffer = malloc(XML_BUFFER_SIZE);


            if (pcaBuffer==NULL)
            {   
                DEBUG(("failed to allocate memory for XML buffer\r\n"));
            }
            else
            {

				// install teh signal handler
				signal(SIGINT, exit_program);

				// do this until we get the exit signal
				while(intExitLoop==0)
				{    
                    int intSize = readXML(intSerialFD, pcaBuffer, XML_BUFFER_SIZE);
    
                    // ok, read the data
                    if (intSize <= 0)
                    {
                        DEBUG(("failed to read XML data\r\n"));
                    }
                    else
                    {
                        if (parseXML(pcaBuffer,  intSize, argv[2] )==0)
                        {
                            DEBUG(("failed to parse the XML data\r\n"));
						}				
                    }
				}

                free(pcaBuffer);
            }

			close(intSerialFD);
    
        }
    




    }

    return(intReturn);
}

